/* C code to read the ds18b20 digital thermometer                     */
/* cabling: when looking onto the small board from the top            */
/* with the pins pointing downwards then                              */
/* left:     Gnd                                                      */
/* middle:   Vcc (3.3V)                                               */
/* right:    data                                                     */
/* we need a 10 kOhms resistor from the data pint to Vcc              */

#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>

FILE *temperatureData;

void intHandler(int dummy) {
  printf("Ctrl C pressed, closing the data file and exiting\n");;
  fclose(temperatureData);
  exit(1);
}    

int main (void) {
  DIR *dir;
  struct dirent *dirent;
  char dev[16];      // Dev ID
  char devPath[128]; // Path to device
  char buf[256];     // Data from device
  char tmpData[6];   // Temp C * 1000 reported by device 
  char path[] = "/sys/bus/w1/devices"; 
  ssize_t numRead;
  

  /* capture Ctrl + C to orderly close the data file before exiting */ 
  signal(SIGINT,intHandler);
	 
  dir = opendir (path);
  if (dir != NULL)
    {
      while ((dirent = readdir (dir)))
	// 1-wire devices are links beginning with 28-
	if (dirent->d_type == DT_LNK && 
	    strstr(dirent->d_name, "28-") != NULL) { 
	  strcpy(dev, dirent->d_name);
	  printf("\nDevice: %s\n", dev);
	}
      (void) closedir (dir);
    }
  else
    {
      perror ("Couldn't open the w1 devices directory");
      return 1;
    }
  
  // Assemble path to OneWire device
  sprintf(devPath, "%s/%s/w1_slave", path, dev);
  // Read temp continuously
  // Opening the device's file triggers new reading
  temperatureData = fopen("temperatureData.txt","w");
  while(1) {
    int fd = open(devPath, O_RDONLY);
    if(fd == -1)
      {
	perror ("Couldn't open the w1 device.");
	return 1;   
      }
    while((numRead = read(fd, buf, 256)) > 0) 
      {
	strncpy(tmpData, strstr(buf, "t=") + 2, 5); 
	float tempC = strtof(tmpData, NULL);
	printf("Device: %s  - ", dev); 
	printf("Temp: %.3f C  ", tempC / 1000);
	printf("%.3f F\n\n", (tempC / 1000) * 9 / 5 + 32);
	fprintf(temperatureData,"%10.4f\n",tempC/1000.0);
	sleep(10);
      }
    close(fd);
  } 
  /* return 0; --never called due to loop */
}
