/*
 * Calculate a moving average filter
 * copyright: U. Raich Dec. 2016
 *
 * This program is part of the workshop on embedded systems held in 2017 th the
 * University of Cape Coast, Ghana
 * It is released under the Gnu Public License
 * You can find the license text here:
 * https://www.gnu.org/licenses/gpl.txt
 */

#include <stdio.h>
#include <getopt.h>
#include <string.h>
#include <stdlib.h>

#define MAX_LENGTH 100

extern char *optarg;
extern int optind, opterr, optopt;

int main(int argc, char *argv[]) {
  
  static struct option long_options[] =
    {
      {"inFile", required_argument, NULL, 'i'},
      {"length", required_argument, NULL, 'l'},
      {"help", no_argument, NULL, 'h'},
      {NULL, 0, NULL, 0}
    };
  char fileName[100]="";
  char outFile[100];
  double *numberArr,*numPtr,*sumPtr,number,sum=0.0,*lastNum;  
  int length=0,ch,i;
  char *pointOut,*pointIn;
  FILE *dataFile,*outDataFile;

  // loop over all of the options
  
  while ((ch = getopt_long_only(argc, argv, "t:a:", long_options, NULL)) != -1)
    {
      // check to see if a single character or long option came through
      switch (ch)
	{
         // short option 'i'
	case 'i':
	  strcpy(fileName,optarg); // or copy it if you want to
	  break;
	  // short option 'l'
	case 'l':
	  length=atoi(optarg); // or copy it if you want to
	  break;
	case '?':
	case 'h':
	default:
	  printf("usage: %s inFile averageLength\n",argv[0]); 
	  break;
	}
    }
  if (!strcmp(fileName,"") || length<2 || length>MAX_LENGTH) {
    printf("usage: %s inFile averageLength\n",argv[0]);
    exit(-1);
  }
  printf("input filename: %s\n",fileName);
  printf("averaging length: %d\n",length);

  /* 
   * create file name for output file from input file name by adding
   * "smoothed"
   */
  strcpy(outFile,fileName);

  if ((pointOut=strrchr(outFile,'.')) == NULL)
    strcat(outFile,"-smoothed");
  else {
    strcpy(pointOut,"-smoothed");
  }

  if ((pointIn=strrchr(fileName,'.')) != NULL)
    strcat(outFile,pointIn);
 
  printf("ouput file name: %s\n",outFile);

  if ((dataFile = fopen(fileName,"r")) == NULL) {
    fprintf(stderr,"Input file not found\n");
    exit(-1);
  }

  if ((numberArr = calloc(length,sizeof(double))) == NULL) {
    fprintf(stderr,"Could not allocate space to calculate sum\n");
    exit(-1);
  };
  numPtr = numberArr;
  for (i=0;i<length;i++) {
    if (fscanf(dataFile,"%lf",&number) == EOF) {
      fprintf(stderr,"Data file is too short\n");
      exit(-1);
    }
    else
      *numPtr++ = number;
  }

  lastNum = numPtr;
  numPtr = numberArr;

  outDataFile = fopen(outFile,"w");
  do {
    if (numPtr==lastNum)           /* cycle through the whole number array */
                                   /* restart at the beginning of the end  */
                                   /* of the array is reached              */
      numPtr = numberArr;
    /* calulate the average over the whole data array */
    sumPtr = numPtr;
    sum=0.0;
    for (i=0;i<length;i++) {
      sum += *sumPtr++;
      if (sumPtr == lastNum)
	sumPtr = numberArr;
    }
    fprintf(outDataFile,"%10.4f\n",sum/(double)length);
  }  while ((fscanf(dataFile,"%lf",numPtr++))!= EOF);
  fclose(outDataFile);
}
